from ML_management.jsonschema_inference import jsonschema_exceptions, jsonschema_inference

SkipJsonSchema = jsonschema_inference.SkipJsonSchema
infer_jsonschema = jsonschema_inference.infer_jsonschema
DictKeysMustBeStringsError = jsonschema_exceptions.DictKeysMustBeStringsError
FunctionContainsVarArgsError = jsonschema_exceptions.FunctionContainsVarArgsError
FunctionContainsVarKwArgsError = jsonschema_exceptions.FunctionContainsVarKwArgsError
InvalidStructureAnnotationError = jsonschema_exceptions.InvalidStructureAnnotationError
NoAnnotationError = jsonschema_exceptions.NoAnnotationError
UnsupportedTypeError = jsonschema_exceptions.UnsupportedTypeError
