__all__ = ["COLLECTORS", "REMOTE_COLLECTORS"]

from ML_management.collectors.collectors import DATA_COLLECTORS, DATA_REMOTE_COLLECTORS

COLLECTORS = DATA_COLLECTORS

REMOTE_COLLECTORS = DATA_REMOTE_COLLECTORS
